import discord
import xfox
from Amisynth.utils import mensaje_id_global


@xfox.addfunc(xfox.funcs)
async def sendMessage(texto, retornar_id="false", canal_id=None, *args, **kwargs): 
    global mensaje_id_global  # Usar la variable global
    
    # Obtener el contexto adecuado
    ctx_message_env = kwargs.get("ctx_message_env")
    ctx_command = kwargs.get("ctx_command")
    ctx_slash_env = kwargs.get("ctx_slash_env")
    ctx_join_member_env = kwargs.get("ctx_join_member_env")
    ctx_remove_member_env = kwargs.get("ctx_remove_member_env")


    # Determinar el canal correcto
    canal = None

    if ctx_command and canal_id:
        canal = ctx_command.bot.get_channel(int(canal_id))  # Obtener canal por ID
    elif ctx_message_env:
        canal = ctx_message_env.guild.get_channel(int(canal_id))
    elif ctx_slash_env:
        canal = ctx_slash_env.guild.get_channel(int(canal_id))
    elif ctx_join_member_env:
        canal = ctx_join_member_env.guild.get_channel(int(canal_id))
    elif ctx_remove_member_env:
        canal = ctx_remove_member_env.guild.get_channel(int(canal_id))

    # Verificar si el canal es válido antes de enviar el mensaje
    if isinstance(canal, discord.TextChannel):
        mensaje = await canal.send(texto)
        if str(retornar_id).lower() == "true":
            mensaje_id_global = mensaje.id  # Guardar el ID del mensaje en la variable global
            return mensaje.id
        
        return ""
    
    print(f"No se encontró un canal válido para enviar el mensaje. Canal ID: {canal_id}")
    return None  # Indicar que falló el envío



import xfox
import discord
from Amisynth.utils import buttons, mensaje_id_global


# Contador de fila global
row_counter = 0  

@xfox.addfunc(xfox.funcs)
async def addButton(new_row: str, button_id: str, label: str, style: str, *args, **kwargs):
    """Crea múltiples botones interactivos y devuelve una lista de objetos de botones creados."""
    if "ctx_command" in kwargs:
        ctx=kwargs["ctx_command"]
    if "ctx_join_member_env" in kwargs:
        ctx=kwargs["ctx_join_member_env"]
    if "ctx_remove_member_env" in kwargs:
        ctx=kwargs["ctx_remove_member_env"]
    if "ctx_message_edit_env" in kwargs:
        ctx=kwargs["ctx_message_edit_env"]
    if "ctx_message_delete_env" in kwargs:
        ctx=kwargs["ctx_message_delete_env"]

    global row_counter  # Para modificar el contador de fila

    # Estilos disponibles
    estilos = {
        "primary": discord.ButtonStyle.primary,
        "cecondary": discord.ButtonStyle.secondary,
        "success": discord.ButtonStyle.success,
        "danger": discord.ButtonStyle.danger,
        "link": discord.ButtonStyle.link
    }
    
    button_style = estilos.get(style, discord.ButtonStyle.primary)

    # Definir valores predeterminados
    disabled = False
    emoji = None
    message_id = None

    if len(args) > 0:
        disabled = args[0].lower() == "true"
    if len(args) > 1:
        emoji = args[1]
    if len(args) > 2:
        message_id = args[2]  # No se está usando en la creación del botón, revisar si es necesario

    # Validar si es un botón de tipo enlace
    custom_id = button_id if button_style != discord.ButtonStyle.link else None
    url = button_id if button_style == discord.ButtonStyle.link else None

    # Lógica para manejar la fila (row)
    if new_row.lower() == "true":
        row_counter += 1  # Aumenta la fila si se indica "true"
    elif new_row.lower() == "re":
        row_counter = 0  # Reinicia la fila si se indica "re"

    button = discord.ui.Button(
        label=label,
        custom_id=custom_id,
        style=button_style,
        emoji=emoji,
        disabled=disabled,
        url=url,
        row=row_counter
    )

    if message_id:
        message_id = int(message_id)  # Convertir a número

        for channel in ctx.guild.text_channels:
            try:
                message = await channel.fetch_message(message_id)  # Obtener el mensaje
                if message.components:
                    view = discord.ui.View.from_message(message)  # Recuperar los botones existentes
                else:
                    view = discord.ui.View()  # Crear una nueva vista si no hay botones
                    view.add_item(button)  # Agregar el botón sin eliminar los anteriores
                    await message.edit(view=view)  # Editar el mensaje con la nueva vista
                    return ""
            except discord.NotFound:
                continue  # Si el mensaje no se encuentra, continuar con el siguiente canal
            except discord.Forbidden:
                continue  # Si no tienes permisos para ver el canal, continuar con el siguiente canal
            except discord.HTTPException as e:
                continue  # Manejar errores HTTP y continuar con el siguiente canal
    
    buttons.append(button)
    return ""


import discord
import re
from discord import app_commands

# Lista global de embeds
embeds = []  # Esta lista se puede actualizar desde otros archivos, como 'image.py' o 'thumbnail.py'

buttons = []  # Si también necesitas manejar botones, puedes añadir la lógica aquí

choices_slash = {}


json_storage = {}


menu_options = {}

mensaje_id_global = None

async def utils():
    nuevos_embeds = []  # Usamos una lista nueva para los embeds procesados

    # Iterar sobre cada item en la lista de embeds

    for item in embeds:
        embed = discord.Embed()  # Crear el embed sin título y descripción por defecto

        if "color" in item and item["color"]:
            embed.color = item["color"]

        # Verificar si el título está presente y agregarlo
        if 'title' in item and item['title']:
            embed.title = item['title']

        if "title_url" in item and item["title_url"]:
            embed.url = ensure_double_slash(item["title_url"] or "")

        # Verificar si la descripción está presente y agregarla
        if 'description' in item and item['description']:
            embed.description = item['description']

        # Verificar si hay imagen y añadirla, asegurando que la URL esté bien formada
        if "image" in item and item["image"]:
            n = ensure_double_slash(item["image"])
            embed.set_image(url=n)

        if "thumbnail_icon" in item and item["thumbnail_icon"]:
            n = ensure_double_slash(item["thumbnail_icon"])
            embed.set_thumbnail(url=n)

        if "footer" in item and item["footer"]:
            icon = item["footer_icon"] if "footer_icon" in item and item["footer_icon"] else None
            embed.set_footer(text=item["footer"], icon_url=icon)
        

        author_name = item.get("author")
        author_icon = ensure_double_slash(item.get("author_icon", ""))
        author_url = ensure_double_slash(item.get("author_url", ""))

        if author_name or author_icon or author_url:
            embed.set_author(name=author_name or "", icon_url=author_icon, url=author_url)
        # Agregar el embed procesado a la lista

        if "fields" in item and isinstance(item["fields"], list):
            for field in item["fields"]:
                embed.add_field(
                    name=field.get("name"),
                    value=field.get("value"),
                    inline=field.get("inline"))


        nuevos_embeds.append(embed)

    # Retornar los nuevos embeds procesados y los botones
    return buttons, nuevos_embeds


def ensure_double_slash(text: str) -> str:
    return re.sub(r"https:(?!//)", "https://", text)






import os
import importlib.util

def load_functions():
    folder = "Amisynth/Functions"
    # Usamos os.walk() para recorrer todos los subdirectorios
    for root, dirs, files in os.walk(folder):
        for filename in files:
            if filename.endswith(".py"):
                # Obtenemos el nombre del módulo sin `.py`
                module_name = filename[:-3]
                # Obtenemos la ruta completa del archivo
                module_path = os.path.join(root, filename)

                # Cargamos y ejecutamos el módulo
                spec = importlib.util.spec_from_file_location(module_name, module_path)
                module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(module)



import xfox

@xfox.addfunc(xfox.funcs)
async def message(num, *args, **kwargs):
    # Filtrar kwargs para ignorar 'ctx_slash_env'
    filtered_kwargs = {k: v for k, v in kwargs.items() if k != "ctx_slash_env"}

    # Obtener el contexto adecuado
    ctx_message_env = kwargs.get("ctx_message_env")
    ctx_command = kwargs.get("ctx_command")

    if ctx_message_env:
        message_content = ctx_message_env.content  # Mensaje del evento
        palabras = message_content.split()  # Obtener palabras
    elif ctx_command and ctx_command.message:
        message_content = ctx_command.message.content  # Mensaje del comando
        palabras = message_content.split()[1:]  # Excluir el nombre del comando
    else:
        palabras = []

    # Si el número es "-1", devolver todo el contenido después del comando
    if num == "-1":
        return " ".join(palabras)  # Devuelve todos los argumentos como un solo string

    # Intentar convertir num a un índice
    try:
        index = int(num) - 1  # Ajuste para que 1 sea el primer elemento
        if 0 <= index < len(palabras):
            palabra = palabras[index]
        else:
            palabra = ""
    except ValueError:
        # Si num no es un número, usarlo como clave de kwargs
        return filtered_kwargs.get(num, "")

    # Si hay argumentos extra, buscar en kwargs si la palabra no existe
    if args:
        for clave in args:
            if clave in filtered_kwargs:
                return palabra if palabra else filtered_kwargs[clave]
    
    return palabra



import discord
from discord.ext import commands
from discord import app_commands
import asyncio
import xfox
import Amisynth.Handler
import Amisynth.utils as utils
from typing import List, Dict, Optional, Any
import os
import importlib.util

# Registrar todas las funciones automáticamente
Amisynth.Handler.register_all()

class AmiClient(commands.Bot):
    def __init__(self, prefix, cogs = None):
        super().__init__(command_prefix=prefix, intents=discord.Intents.all())
        self._cogs = cogs  # Usar una variable interna para los cogs
        self.comandos_personalizados = {}
        self.eventos_personalizados = {
            "$onMessage": [],
            "$onReady": [],
            "$onReactionAdd": [],  # Evento de agregar reacción
            "$onReactionRemove": [],  # Evento de remover reacción
            "$onInteraction": [],
            "$onMessageEdit": [],
            "$onMessageDelete": [],
            "$onJoinMember": [],
            "$onLeaveMember": []
        }
    
    async def setup_hook(self):
        """Cargar todos los cogs de forma asincrónica."""
        if self._cogs:  # Verificar si se pasó una carpeta de cogs
            await self.load_cogs(self._cogs)

    async def load_cogs(self, carpeta):
        """Cargar cogs de forma asincrónica."""
        for filename in os.listdir(carpeta):
            if filename.endswith(".py"):
                cog_path = os.path.join(carpeta, filename)

                # Cargar módulo dinámicamente
                spec = importlib.util.spec_from_file_location(filename[:-3], cog_path)
                module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(module)

                # Si el cog tiene una función setup(bot), la ejecutamos
                if hasattr(module, "setup"):
                    await module.setup(self)  # Ejecutar setup(bot) si es async


    def new_command(self, name, type, code):
        async def custom_command(ctx_command):
            kwargs = {"ctx_command": ctx_command}  # Pasar ctx renombrado
            result = await xfox.parse(code, del_empty_lines=True, **kwargs)
        
            texto = result
            botones, embeds = await utils.utils()
            # Construir el View si hay botones
            view = discord.ui.View()
            if botones:
                for boton in botones:  # Extraer la fila y el botón
                    view.add_item(boton)

            

            # Enviar mensaje con el tipo adecuado
            await ctx_command.send(
                content=texto if texto else None,  # Si hay texto, se agrega
                view=view if botones else None,    # Si hay botones, se agrega el View
                embeds=embeds if embeds else None  # Si hay embeds, se agregan
            )



        self.comandos_personalizados[name] = {"type": type, "code": code}
        self.add_command(commands.Command(custom_command, name=name))
    
    
    def new_slash(
        self,
        name: str,
        description: str,
        code: str = "",
        options: Optional[List[Dict[str, Any]]] = None
    ):
        parameters = ["interaction: discord.Interaction"]
        choices_kwargs = {}

        if options:
           for option in options:
                option_name = option.get("name_option")
                param_name = option_name.replace(" ", "_")  # ⚠️ Corrección aquí
                option_type = option.get("tipo", "str")
                option_required = option.get("required", False)
                if option_required == False:
                    parameters.append(f"{param_name}: {option_type} = None")  # Hace que sean opcionales
                else: 
                    parameters.append(f"{param_name}: {option_type}")  # Hace que sean opcionales


                if "choices" in option and isinstance(option["choices"], list):
                    choices_kwargs[param_name] = {
                        choice["name_choice"]: choice["value_choice"]
                        for choice in option["choices"]
                    }

        params_str = ", ".join(parameters)

        func_code = f"""async def slash_command({params_str}):
        kwargs = {{"ctx_slash_env": interaction}}

        for key in {list(choices_kwargs.keys())} + {list(set([opt["name_option"].replace(" ", "_") for opt in options] if options else []))}:
            value = locals().get(key, None)

            if value is not None:
                choice_data = {choices_kwargs}.get(key)
                if choice_data and value in choice_data:
                    kwargs[key] = choice_data[value]
                else:
                    kwargs[key] = value

        result = await xfox.parse({repr(code)}, del_empty_lines=True, **kwargs)

        texto = result
        botones, embeds = await utils.utils()
        view = discord.ui.View()
        if botones:
            for boton in botones:
                view.add_item(boton)

        await interaction.response.send_message(
            content=texto if texto else None,
            view=view,
            embeds=embeds if embeds else [],
            ephemeral=False
        )"""

        exec(func_code, globals(), locals())
        command_func = locals()["slash_command"]

        decorated_func = self.tree.command(name=name, description=description)(command_func)

        for key, choices_dict in choices_kwargs.items():
            decorated_func = app_commands.choices(**{
                key: [app_commands.Choice(name=name, value=value) for name, value in choices_dict.items()]
            })(decorated_func)

        self.comandos_personalizados[name] = {"type": "slash", "code": code}





    def new_event(self, tipo, codigo, overwrite=False):
        if tipo not in self.eventos_personalizados or overwrite:
            self.eventos_personalizados[tipo] = []  # Reiniciar si se sobrescribe
        self.eventos_personalizados[tipo].append(codigo)

    async def ejecutar_eventos(self, tipo, 
                               ctx_message_env=None, 
                               ctx_reaction_env=None, 
                               ctx_reaction_remove_env=None, 
                               ctx_interaction_env=None, 
                               ctx_message_edit_env=None, 
                               ctx_message_delete_env=None,
                               ctx_join_member_env=None,
                               ctx_remove_member_env=None):
        

        if tipo in self.eventos_personalizados:
            for codigo in self.eventos_personalizados[tipo]:
                kwargs = {
                    "ctx_message_env": ctx_message_env,
                    "ctx_reaction_env": ctx_reaction_env,
                    "ctx_reaction_remove_env": ctx_reaction_remove_env,
                    "ctx_interaction_env": ctx_interaction_env,  # 👈 Agregado aquí
                    "ctx_message_edit_env": ctx_message_edit_env,
                    "ctx_message_delete_env": ctx_message_delete_env,
                    "ctx_join_member_env": ctx_join_member_env,
                    "ctx_remove_member_env": ctx_remove_member_env
                }
                result = await xfox.parse(codigo, del_empty_lines=True, **kwargs)
                botones, embeds = await utils.utils()
                view=None
                if botones:
                    # Crear un View para los botones
                    view = discord.ui.View()
                    for boton in botones:
                        view.add_item(boton)  # Agregar los botones al View

                if ctx_message_env:
                    await ctx_message_env.channel.send(result, 
                                                        view=view if view else None,  
                                                        embeds=embeds if embeds else [])

                elif ctx_reaction_env:
                    channel = self.get_channel(ctx_reaction_env.channel_id, )
                    if channel:
                        await channel.send(result, 
                                            view=view if view else None,
                                            embeds=embeds if embeds else [])


                elif ctx_reaction_remove_env:
                    channel = self.get_channel(ctx_reaction_remove_env.channel_id)
                    if channel:
                        await channel.send(result, 
                                            view=view,
                                            embeds=embeds if embeds else [])

                elif ctx_interaction_env:
                    await ctx_interaction_env.response.edit_message(content=result, 
                                                                    view=view,
                                                                    embeds=embeds)

                elif ctx_message_edit_env:
                    before, after = ctx_message_edit_env
                    await before.channel.send(content=result, 
                                              view=view, 
                                              embeds=embeds)

                elif ctx_message_delete_env:
                    await ctx_message_delete_env.channel.send(content=result, 
                                                                    view=view,
                                                                    embeds=embeds)


    async def on_message(self, ctx_message_env):
        if ctx_message_env.author.bot:
            return
        
        await self.ejecutar_eventos("$onMessage", ctx_message_env)
        await self.process_commands(ctx_message_env)  # Permite que otros comandos de discord.py sigan funcionando




    async def on_ready(self):
        print(f"Bot conectado como {self.user}")
        await self.ejecutar_eventos("$onReady")
        try:
            synced = await self.tree.sync()
            
        except Exception as e:
            print(f"Error al sincronizar slash commands: {e}")

    async def on_member_join(self, member: discord.Member):
        await self.ejecutar_eventos("$onJoinMember", ctx_join_member_env=member)
    
    async def on_member_remove(self, member: discord.Member):
        await self.ejecutar_eventos("$onLeaveMember", ctx_remove_member_env=member)


        
    async def on_raw_reaction_add(self, ctx_reaction_env):

        """Maneja cuando un usuario añade una reacción."""
        await self.ejecutar_eventos("$onReactionAdd", ctx_reaction_env=ctx_reaction_env)

    async def on_raw_reaction_remove(self, ctx_reaction_remove_env):
        """Maneja cuando un usuario remueve una reacción."""
        await self.ejecutar_eventos("$onReactionRemove", ctx_reaction_remove_env=ctx_reaction_remove_env)

    async def on_interaction(self, ctx_interaction_env: discord.Interaction):
        """Maneja interacciones como botones y menús."""
        if ctx_interaction_env.user.bot:
            return
    
        await self.ejecutar_eventos("$onInteraction", ctx_interaction_env=ctx_interaction_env)

    async def on_message_edit(self, before, after):
        if before.author.bot:  # Evita que el bot procese sus propios mensajes
            return
        await self.ejecutar_eventos("$onMessageEdit", ctx_message_edit_env=(before, after))
        
    async def on_message_delete(self, ctx_message_delete_env):
        if ctx_message_delete_env.author.bot:
            return
        await self.ejecutar_eventos("$onMessageDelete", ctx_message_delete_env=ctx_message_delete_env)



import xfox
import discord

@xfox.addfunc(xfox.funcs)
async def customID(*args, **kwargs):
    if "ctx_interaction_env" in kwargs:
        interaction = kwargs["ctx_interaction_env"]
        return interaction.data.get("custom_id")
    
    return ""
